"""
Procuret Python
Human Identity Document Type Module
author: hugh@blinkybeach.com
"""
from enum import Enum


class HumanIdentityDocumentType(Enum):

    AUS_DRIVER_LICENSE = 1
    NZ_DRIVER_LICENSE = 2
