"""
Procuret Python
Session Perspective Module
author: hugh@blinkybeach.com
"""
from enum import Enum


class Perspective(Enum):
    SUPPLIER = 1
    BUSINESS = 2
    ADMINISTRATOR = 3
    SALES = 4
    TREASURY = 5
    INVESTOR = 6
    CUSTOMER_SUPPORT = 7
